/* ************************************************************************** */
// Programmer: Tobin Schwaiger-Hastanan (tobin@13hex.com)
// 
// Copyright (C) 2003 13Hex, Inc.
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions 
// are met:
// 
//      * Redistributions of source code must retain the above copyright 
//        notice, this list of conditions and the following disclaimer.
// 
//      * Redistributions in binary form must reproduce the above 
//        copyright notice, this list of conditions and the following 
//        disclaimer in the documentation and/or other materials 
//        provided with the distribution. 
// 
//      * Neither the name of 13Hex, Inc. nor the names 
//        of its contributors may be used to endorse or promote 
//        products derived from this software without specific prior 
//        written permission. 
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
// COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
// POSSIBILITY OF SUCH DAMAGE.
/* ************************************************************************** */
#ifndef _ENTITYMANAGER_DOT_H_INCLUDED_
#define _ENTITYMANAGER_DOT_H_INCLUDED_

namespace snbfrx
{
    /**
      * Singleton EntityManager used to manage EntityFactory and Entity classes.
      *
      * This calls allows you to manage EntityFactory classes by name.  You can
      * look up and get name factories classes.  The factories are used to instantiate
      * entity objects.  Entities are any dynamic elements that may be in your game that
      * need to be managed through a simple update and render methods.
      *
      * When you add an entity 
      */
    class EntityManager
    {
    private:
        /**
          * Default constructor, private to protect the singleton implementation.
          */
        EntityManager();
        
        /**
          * Unimplemented copy constructor to keep instances of EntityManager objects 
          * from being copied.
          */
        EntityManager( const EntityManager& );
    public:

        /**
          * Virtual destructor.
          */
        virtual ~EntityManager();

        /**
          * Singleton instantiation interface.  This method returns
          * a single instance of an entity manager.
          *
          * @return Pointer to an instantiated Entity Manager.
          */
        static EntityManager* getInstance();
        
        /**
          * Determines if the EntityManager already has a factory registered
          * under a specific name.
          * 
          * @param name string object containing the name to look up.
          * @return bool value specifying if a factory exists by a given name.
          */
        bool hasFactory( const std::string& name ) const;

        /**
          * Determines if factory object has already been registered in the EntityManager.          
          * 
          * @param pfactory pointer to an EntityFactory to check existance of.
          * @return bool value specifying if a factory exists.
          */
        bool hasFactory( EntityFactory* pfactory ) const;

        /**
          * Registers a factory under a specific name.
          *
          * @param name name of factory being registered.
          * @param pfactory point to factory to be register.
          *
          * @return pointer to the factory added.  If null, factory could not be added. 
          *            user should check to see if a factory exists of the same name or the 
          *         factory object has already be registered.
          */
        EntityFactory* registerFactory( const std::string& name, EntityFactory* pfactory );

        /**
          * Gets a factory of a specific name.
          *
          * @param name name of factory to get.
          * @return pointer to Entity Factory.  If the factory does not exist, 
          *         null is returned.
          */
        EntityFactory* getFactory( const std::string& name ) const;
        /**
          * Erases a registered entity factory from the manager.
          * This method does not do deletion of the object.
          * 
          * @param name name of the factory to erase.
          * @return point to the factory removed from the manager.
          */
        EntityFactory* eraseFactory( const std::string& name );

        /**
          * Gets all registered factories assigned to the manager.
          *
          * @return a set of pointers to Entity factories.
          */
        const std::set<EntityFactory*>& getFactories() const { return m_factorySet; }


        /**
          * Checks to see if an active entity is already being managed.
          * 
          * @param pentity pointer to entity to check.
          * @return bool value specifying if entity exists or not.
          *         true  - entity exists.
          *         false - entity does not exist.
          */
        bool hasEntity( Entity* pentity ) const;

        /**
          * Adds an entity to the manager.
          *
          * NOTE: only unique entities may be managed, 
          *       two of the same objects cannot be added.
          * 
          * @param pentity pointer to entity object to be added.
          * @return pointer to entity added. If entity already is managed,
          *         null is returned.
          */
        Entity* addEntity( Entity* pentity );

        /**
          * Erases entity from manager.
          * This method does not release the entity, see the update method
          * for more info about releasing entities.
          * 
          * @param pentity pointer to entity to be removed.
          * @return pointer to entity removed from manager.
          */
        Entity* eraseEntity( Entity* pentity );
        
        /**
          * Gets all active entities managed by manager.
          * 
          * @return set of points to active entity objects.
          */
        const std::set<Entity*>& getEntities() const { return m_entitySet; }

        /**
          * Updates all active entities.
          */
        void update();

        /**
          * Renders all active entities.
          */
        void render();        
    private:

        /** static pointer to an instance of the Entity Manager. */
        static EntityManager* m_pinstance;

        /** map of registered entity factories */
        std::map<std::string, EntityFactory*> m_factoryMap;

        /** set of registered factories */
        std::set<EntityFactory*> m_factorySet;

        /** set of active entities */
        std::set<Entity*> m_entitySet;
    };
}

#endif